%% Draft Model Reduced
% author: Victor A. Lopez-Agudelo, University of Valle
clc;
clear all;
%addpath(genpath('G:\Mi unidad\0_POSTDOC-UNIVALLE\RAVEN-master'))
% Carga el modelo de  S. clavuligerus. Descargado de: 
% https://www.mdpi.com/2076-2607/8/9/1255
% excel file containing the model with the subsystems we want to include in the reduced model.
% load excel files for each model reduction iteration: sclav_red_0,sclav_red_1, sclav_red_2, sclav_red_3...

filename = 'reduced_sclav_rounds/sclav_red_draft_%d.mat';

inputask = inputdlg('Enter number of reduced model iteration: example inputs 0,1,2,3,...',...
                    'Sclav Reduced Model',[1 20])

user_input = str2double(inputask{1}); %converting to double user input
                
%modelSclav    = importExcelModel(sprintf(filename,user_input),true); 
load(sprintf(filename,user_input)); 

modelSclav.id     = 'sclav_atcc_27064';

% Save data for display:
i_rxns = length(modelSclav.rxns);
i_mets = length(modelSclav.mets);
i_genes = length(modelSclav.genes);

%% Agregar datos acerca de S.clavuligerus

modelSclav.annotation.defaultLB    = -1000;
modelSclav.annotation.defaultUB    = +1000;
modelSclav.annotation.taxonomy     = 'taxonomy/1901';
modelSclav.annotation.givenName    = 'Victor A.';
modelSclav.annotation.familyName   = 'Lpez-Agudelo';
modelSclav.annotation.email        = 'valonso.lopez@udea.edu.co';
modelSclav.annotation.organization = 'Universidad del Valle';
modelSclav.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
modelSclav.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';

%% Identify dead end metabolites and zero flux reactions

SM = full(modelSclav.S);
single_connected = zeros(length(modelSclav.mets),1);
dead_end = zeros(length(modelSclav.mets),1);
blocked_reaction_list = zeros(length(modelSclav.rxns),1);

count_mets = 0;
count_rxns = 0;

for i = 1:length(modelSclav.mets)
    
   id_reaction = find(SM(i,:));
   n_rxns = length(id_reaction);
   id_neg = find(SM(i,:)<0);
   id_pos = find(SM(i,:)>0);
   neg_empty = isempty(id_neg);
   pos_empty = isempty(id_pos);
   
   if (n_rxns == 1) || (neg_empty == 1 && pos_empty == 0)
      
       if (n_rxns == 1)
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       single_connected(count_mets) = i;
       blocked_reaction_list(count_rxns) = id_reaction;
       
       else 
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       dead_end (count_mets) = i;
       
            for z = 1:length(id_reaction)
           
            blocked_reaction_list(count_rxns+z) = id_reaction(z);
       
            end
            
       count_rxns = count_rxns + z;
   
       end
       
   end
    
end

single_connected(single_connected == 0) = []; % List of single connected metabolite ids
dead_end(dead_end == 0) = [];                 % List of dead end metabolite ids
blocked_reaction_list(blocked_reaction_list == 0) = []; % List of blocked reactions ids
blocked_reaction_list = unique(blocked_reaction_list); % List of unique blocked reactions ids
sc_mets = modelSclav.mets(single_connected); % Vector list of single connected metabolites
de_mets = modelSclav.mets(dead_end) ;% Vector List of dead end metabolites
brxns = modelSclav.rxns(blocked_reaction_list); % vector list of blocked reactions

all_sc_de = modelSclav.mets(union(single_connected,dead_end)); % list of single connected and dead end metabolite

%% Removing single connected and dead end mets from network // removing blocked reactions associated with single connected mets and dead end mets.

% Removing blocked reactions associated with single connected mets and dead end mets.
modelSclav.rxns(blocked_reaction_list) = [];
modelSclav.S(:,blocked_reaction_list) = [];
modelSclav.lb(blocked_reaction_list) = [];
modelSclav.ub(blocked_reaction_list) = [];
modelSclav.rev(blocked_reaction_list) = [];
modelSclav.c(blocked_reaction_list) = [];
modelSclav.rxnNames(blocked_reaction_list) = [];
modelSclav.grRules(blocked_reaction_list) = [];
modelSclav.rxnGeneMat(blocked_reaction_list,:) = [];
modelSclav.subSystems(blocked_reaction_list) = [];
modelSclav.eccodes(blocked_reaction_list) = [];
modelSclav.rxnMiriams(blocked_reaction_list) = [];
modelSclav.rxnNotes(blocked_reaction_list) = [];
modelSclav.rxnConfidenceScores(blocked_reaction_list) = [];
modelSclav.rxnReferences(blocked_reaction_list) = [];


% removing single connected and dead end metabolites

non_conected = zeros(length(modelSclav.genes),1);
count_nc = 0;

for y = 1: length(modelSclav.genes)
    
    nc_met = find(modelSclav.S(y,:));
    true_false = isempty(nc_met);
    
    if true_false == 1
    count_nc= count_nc + 1;
    non_conected(count_nc) = y;    % saving the position of non-conected metabolites in the stoichiometric matrix (zero s coefficients)
    end
    
end

non_conected (non_conected == 0) = [];

modelSclav.mets(non_conected) = [];
modelSclav.S(non_conected,:) = [];
modelSclav.metNames(non_conected) = [];
modelSclav.metComps(non_conected) = [];
modelSclav.metFormulas(non_conected) = [];
modelSclav.metMiriams(non_conected) = [];
modelSclav.metCharges(non_conected) = [];
modelSclav.b(non_conected) = [];

% removing non-associated genes in GPRs of reactions in the reduced model

na_gprs = zeros(length(modelSclav.genes),1);
count_g = 0;
for x = 1: length(modelSclav.genes)
    
    rxn_2_gene = find(modelSclav.rxnGeneMat(:,x));
    true_false = isempty(rxn_2_gene);
    
    if true_false == 1
    count_g = count_g + 1;
    na_gprs(count_g) = x;    % saving the position of non-associated genes in GPRs of reactions
    end
    
end

na_gprs(na_gprs == 0) = [];
non_associated2gpr_genes = modelSclav.genes(na_gprs);

% removing non-associated genes to reactions in the reduced model

modelSclav.rxnGeneMat(:,na_gprs) = [];
modelSclav.genes(na_gprs) = [];
modelSclav.geneMiriams(na_gprs) = [];
modelSclav.geneShortNames(na_gprs) = [];


%% Saving Model
user_input = user_input + 1;
% .mat format
filemodelmat = 'reduced_sclav_rounds/sclav_red_draft_%d.mat';
save(sprintf(filemodelmat,user_input),'modelSclav');

% .xml format
%exportModel(modelSclav,'sclav_red_1.xml');

% .xlsx format
%exportToExcelFormat(modelSclav,sprintf(filename,user_input));


% saving dead_ends, single connected and blocked reactions

save(sprintf('reduced_sclav_rounds/dead_end_%d.mat',user_input),'de_mets');
save(sprintf('reduced_sclav_rounds/single_connected_%d.mat',user_input),'sc_mets');
save(sprintf('reduced_sclav_rounds/blocked_reactions_%d.mat',user_input),'brxns');

% Display results of reduction
z_rxns = length(modelSclav.rxns);
z_mets = length(modelSclav.mets);
z_genes = length(modelSclav.genes);

user_input_minus = user_input - 1;

fprintf('sclav_red_%d : %d reactions, %d metabolites and %d genes\n',user_input_minus,i_rxns,i_mets,i_genes)
fprintf('sclav_red_%d : %d reactions, %d metabolites and %d genes\n',user_input,z_rxns,z_mets,z_genes)