%% Draft Model Reduced
% author: Victor A. Lopez-Agudelo, University of Valle
clc;
clear all;
%addpath(genpath('G:\Mi unidad\0_POSTDOC-UNIVALLE\RAVEN-master'))
% Carga el modelo de  S. clavuligerus. Descargado de: 
% https://www.mdpi.com/2076-2607/8/9/1255
% excel file containing the model with the subsystems we want to include in the reduced model.
% load excel files for each model reduction iteration: sclav_red_0,sclav_red_1, sclav_red_2, sclav_red_3...

filename = 'sclav_red_draft_%d.xlsx';

inputask = inputdlg('Enter number of reduced model iteration: example inputs 0,1,2,3,...',...
                    'Sclav Reduced Model',[1 20])

user_input = str2double(inputask{1}); %converting to double user input
                
%modelSclav    = importExcelModel(sprintf(filename,user_input),true); 
sclav_red = importExcelModel(sprintf(filename,user_input),true); 

%% Agregar datos acerca de S.clavuligerus
sclav_red.id     = 'sclav_atcc_27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'Lpez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'Lpez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';

%% Solution FBA
SolverOK = changeCobraSolver('glpk','LP');

exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
model = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % define medium unlimited

solutionFBA = optimizeCbModel(model,'max');

% if (solutionFBA.f > 0) || (solutionFBA.stat == -1)
    
    if (solutionFBA.f == 0)
    fprintf('SCLAV REDUCED MODEL CAN NOT PRODUCE BIOMASS COMPONENTS. PLEASE CHECK BIOMASS PRECURSORS \n')
    else
    fprintf('SCLAV REDUCED MODEL CAN NOT GROW DUE TO GLPK issue-No solution reported (timelimit, numerical problem etc \n')
    end
% Chequedo de produccin de precursores de biomasa
% En los FBA anteriores, sclav_red no pudo crecer en medio. Esto indica
% que posiblemente existen metabolitos o precursores de biomasa que no
% pueden ser producidos en ese medio. 

exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
sclav_red2 = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % define medium unlimited

%Lista de metabolitos que estn en la funcin objetivo de biomasa 'Growth'
Growth_precursors_FO = sclav_red2.mets(find(full(sclav_red2.S(:,findRxnIDs(sclav_red2,'Growth')))));
n_mets = length(Growth_precursors_FO); % nmero de metabolitos en la funcin objetivo

met_production = zeros(n_mets,1);% valor de produccion de precursores de biomasa.

for i = 1:n_mets
 % Agrego DemandRxns para cada uno de los precursores de biomasa
 % Precursores necesarios para crecimiento ilimitado
 model__DM = addDemandReaction(sclav_red2, Growth_precursors_FO(i));
 model__DM = changeObjective(model__DM,sprintf('DM_%s',Growth_precursors_FO{i}),1);
 FBA__DM = optimizeCbModel(model__DM,'max');
 met_production(i) = FBA__DM.f;
 
 end
 
 PrecursoresProducidos = table(Growth_precursors_FO,met_production,...
     'VariableName',{'Biomass_Precursors','Production_in_unlimited_medium'})
 
 met_noproducidos = Growth_precursors_FO(find(~met_production));
 
 

% else
    exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
    model = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % define medium unlimited
    
    solutionFBA = optimizeCbModel(model,'max');
    fprintf(sprintf('SCLAV REDUCED MODEL IS GROWING AT A SPECIFIC GROWTH RATE OF %d 1/h \n',solutionFBA.f))
           
% end


%% clavulanic acid can be produced ?. 

exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
model_DM_ca = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % exchange reactions
model_DM_ca = changeRxnBounds(model_DM_ca,{'EX_clav_e','EX_ceph_e','EX_cephal_e','EX_6APA_e',...
                                           'EX_gclavam_e','EX_acclavam_e','EX_acgclavam_e',...
                                           'EX_clav5s_e','EX_alaclav_e','EX_carbox_e',...
                                           'EX_clavcar_e'},0,'l');
model_DM_ca = addDemandReaction(model_DM_ca, 'clava[e]');
model_DM_ca = changeObjective(model_DM_ca,sprintf('DM_%s','clava[e]'),1);
FBA_DM_ca = optimizeCbModel(model_DM_ca,'max');
CA_production = FBA_DM_ca.f;
fprintf(sprintf('SCLAV REDUCED MODEL IS PRODUCING CLAVULANIC ACID IN UNLIMITED MEDIUM AT A RATE OF %d mmol/gr*h \n',CA_production))

%% Identify dead end metabolites and zero flux reactions

SM = full(sclav_red.S);
single_connected = zeros(length(sclav_red.mets),1);
dead_end = zeros(length(sclav_red.mets),1);
blocked_reaction_list = zeros(length(sclav_red.rxns),1);

count_mets = 0;
count_rxns = 0;

for i = 1:length(sclav_red.mets)
    
   id_reaction = find(SM(i,:));
   n_rxns = length(id_reaction);
   id_neg = find(SM(i,:)<0);
   id_pos = find(SM(i,:)>0);
   neg_empty = isempty(id_neg);
   pos_empty = isempty(id_pos);
   
   if (n_rxns == 1) || (neg_empty == 1 && pos_empty == 0)
      
       if (n_rxns == 1)
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       single_connected(count_mets) = i;
       blocked_reaction_list(count_rxns) = id_reaction;
       
       else 
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       dead_end (count_mets) = i;
       
            for z = 1:length(id_reaction)
           
            blocked_reaction_list(count_rxns+z) = id_reaction(z);
       
            end
            
       count_rxns = count_rxns + z;
   
       end
       
   end
    
end

single_connected(single_connected == 0) = []; % List of single connected metabolite ids
dead_end(dead_end == 0) = [];                 % List of dead end metabolite ids
blocked_reaction_list(blocked_reaction_list == 0) = []; % List of blocked reactions ids
blocked_reaction_list = unique(blocked_reaction_list); % List of unique blocked reactions ids
sc_mets = sclav_red.mets(single_connected); % Vector list of single connected metabolites
de_mets = sclav_red.mets(dead_end) ;% Vector List of dead end metabolites
brxns = sclav_red.rxns(blocked_reaction_list); % vector list of blocked reactions

all_sc_de = sclav_red.mets(union(single_connected,dead_end)); % list of single connected and dead end metabolite


%% Flux Variability Analysis
exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
modelFVA= changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % exchange reactions
modelFVA = changeRxnBounds(modelFVA,{'EX_clav_e','EX_ceph_e','EX_cephal_e','EX_6APA_e',...
                                           'EX_gclavam_e','EX_acclavam_e','EX_acgclavam_e',...
                                           'EX_clav5s_e','EX_alaclav_e','EX_carbox_e',...
                                           'EX_clavcar_e'},0,'l');
[minFVA,maxFVA] = fluxVariability(modelFVA,100);

fva_zero_flux_rxns_id = zeros(length(modelFVA.rxns),1);

for k = 1:length(modelFVA.rxns)
    
    if (minFVA(k) == 0) && (maxFVA(k) == 0)
        
        fva_zero_flux_rxns_id(k) = k;
        
    end
    
end

fva_zero_flux_rxns_id(fva_zero_flux_rxns_id == 0) = [];
fva_zero_flux_rxns = modelFVA.rxns(fva_zero_flux_rxns_id); % List of zero flux reactions based on FVA