%% Draft Model Reduced
% author: Victor A. Lopez-Agudelo, University of Valle
clc;
%clear all;

SolverOK = changeCobraSolver('ibm_cplex','all');
%addpath(genpath('G:\Mi unidad\0_POSTDOC-UNIVALLE\RAVEN-master'))
% Carga el modelo de  S. clavuligerus. Descargado de: 
% https://www.mdpi.com/2076-2607/8/9/1255
% excel file containing the model with the subsystems we want to include in the reduced model.
% load excel files for each model reduction iteration: sclav_red_0,sclav_red_1, sclav_red_2, sclav_red_3...

filename = 'sclav_red_draft_%d.xlsx';

inputask = inputdlg('Enter number of reduced model iteration: example inputs 0,1,2,3,...',...
                    'Sclav Reduced Model',[1 20])

user_input = str2double(inputask{1}); %converting to double user input
                
%modelSclav    = importExcelModel(sprintf(filename,user_input),true); 
sclav_red = importExcelModel(sprintf(filename,user_input),true); 

%% Agregar datos acerca de S.clavuligerus
sclav_red.id     = 'sclav_atcc_27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'Lpez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'Lpez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';


exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red));
model = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l');
solutionFBA = optimizeCbModel(model,'max');
printFluxVector(model, solutionFBA.x, true, true)


%% Chequedo de produccin de precursores de biomasa
% En los FBA anteriores, sclav_red no pudo crecer en medio. Esto indica
% que posiblemente existen metabolitos o precursores de biomasa que no
% pueden ser producidos en ese medio. 

% exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
% sclav_red2 = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % define medium unlimited

%Lista de metabolitos que estn en la funcin objetivo de biomasa 'Growth'
Growth_precursors_FO = model.mets(find(full(model.S(:,findRxnIDs(model,'Growth')))));
n_mets = length(Growth_precursors_FO); % nmero de metabolitos en la funcin objetivo

met_production = zeros(n_mets,1);% valor de produccion de precursores de biomasa.

for i = 1:n_mets
 % Agrego DemandRxns para cada uno de los precursores de biomasa
 % Precursores necesarios para crecimiento ilimitado
 model__DM = addDemandReaction(model, Growth_precursors_FO(i));
 model__DM = changeObjective(model__DM,sprintf('DM_%s',Growth_precursors_FO{i}),1);
 FBA__DM = optimizeCbModel(model__DM,'max');
 met_production(i) = FBA__DM.f;
 
 end
 
 PrecursoresProducidos = table(Growth_precursors_FO,met_production,...
     'VariableName',{'Biomass_Precursors','Production_in_unlimited_medium'})
 
 met_noproducidos = Growth_precursors_FO(find(~met_production));
 
 

% else
    %exchangeRxns = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
    %model = changeRxnBounds(sclav_red,exchangeRxns,-1000,'l'); % define medium unlimited
    
  
           
% end




%% Identify dead end metabolites and zero flux reactions

SM = full(sclav_red.S);
single_connected = zeros(length(sclav_red.mets),1);
dead_end = zeros(length(sclav_red.mets),1);
blocked_reaction_list = zeros(length(sclav_red.rxns),1);

count_mets = 0;
count_rxns = 0;

for i = 1:length(sclav_red.mets)
    
   id_reaction = find(SM(i,:));
   n_rxns = length(id_reaction);
   id_neg = find(SM(i,:)<0);
   id_pos = find(SM(i,:)>0);
   neg_empty = isempty(id_neg);
   pos_empty = isempty(id_pos);
   
   if (n_rxns == 1) || (neg_empty == 1 && pos_empty == 0)
      
       if (n_rxns == 1)
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       single_connected(count_mets) = i;
       blocked_reaction_list(count_rxns) = id_reaction;
       
       else 
           
       count_mets = count_mets + 1;
       count_rxns = count_rxns + 1;
       dead_end (count_mets) = i;
       
            for z = 1:length(id_reaction)
           
            blocked_reaction_list(count_rxns+z) = id_reaction(z);
       
            end
            
       count_rxns = count_rxns + z;
   
       end
       
   end
    
end

single_connected(single_connected == 0) = []; % List of single connected metabolite ids
dead_end(dead_end == 0) = [];                 % List of dead end metabolite ids
blocked_reaction_list(blocked_reaction_list == 0) = []; % List of blocked reactions ids
blocked_reaction_list = unique(blocked_reaction_list); % List of unique blocked reactions ids
sc_mets = sclav_red.mets(single_connected); % Vector list of single connected metabolites
de_mets = sclav_red.mets(dead_end) ;% Vector List of dead end metabolites
brxns = sclav_red.rxns(blocked_reaction_list); % vector list of blocked reactions

all_sc_de = sclav_red.mets(union(single_connected,dead_end)); % list of single connected and dead end metabolite


%% Identifying Unbounded Reactions in Sclav Model
% 12 / 06 / 2019
% This methodology allows to identify Thermodynamically infeasible cycles
% It was adapted from the mathematical description of Schellenberger paper :
% Schellenberger, J., Lewis, N. E., & Palsson, B. ?. (2011). 
% Elimination of thermodynamically infeasible loops in steady-state metabolic models.
% Biophysical journal, 100(3), 544-553.
% Victor A. Lopez-Agudelo - University of Antioquia - Colombia
 

model = changeRxnBounds(sclav_red,exchangeRxns,-1,'l');

METABOLIC_MODEL = model; % model's names

        
NURs = zeros(length(METABOLIC_MODEL), 2);

%solverOK = changeCobraSolver('gurobi7','LP');

%% Counting and Identifying Unbounded Reactions
    
model = METABOLIC_MODEL;  

[minFVA,maxFVA] = fluxVariability(model,100); % FVA 
    
count_minFVA = length(minFVA(minFVA < -50));
count_maxFVA = length(maxFVA(maxFVA > 50));

count_minmax = 0;

%% Unbounded Rxns
URs_max = model.rxns(maxFVA > 50);
URs_min = model.rxns(minFVA < -50);
URs = union(URs_max,URs_min);
%% positions of Unbounded Rxns
positions_vector = (1:1:length(model.rxns))';
id_URs_max = positions_vector(maxFVA > 50);
id_URs_min = positions_vector(minFVA < -50);
id_URs = union(id_URs_max, id_URs_min); % positions of URs
%% Stoichiometric Matrix of Unbounded Reactions with metabolites and reactions
MatrixS = full(model.S);
MatrixS = MatrixS(:,id_URs); % Stoichiometric Matrix of Unbounded Reactions
%% positions of Metabolites
mets_positions_vector = zeros(length(model.mets),1);
for q = 1:length(model.mets)   
    find_met_vector = find(model.S(q,:)); % 
    empty = isempty(find_met_vector); % is empty "find_met_vector"?.
    if (empty == 0)
        mets_positions_vector(q) = q;
    end
end 
mets_positions_vector(mets_positions_vector == 0) = [];
MatrixS = MatrixS(mets_positions_vector,:);
NullSpace = null(MatrixS,'r'); % vectors that form the null space of the S matrix.
UBrxnList = model.rxns(id_URs); % Unbounded Reaction list sorted
[mrow,ncol] = size(NullSpace);
% Finding the LB and UB for the Unbounded Reactions
model.lb = model.lb(id_URs);
model.ub = model.ub(id_URs);

%% writing text in file with possible TICs
folder = cd;  % 
filename = fullfile(folder, sprintf('TICs_%s.txt', 'sclav_red')); % create file with Thermodynamical infeasible cycles
fileID = fopen(filename, 'a');
if fileID == -1, error('Cannot open file: %s', filename); end

TIC_count = 0; % TIC counter
 for z = 1:ncol
     
     fprintf(fileID, '###########\n'); % No. of TIC
     rxncount = 0; % counter of added reaction in List
     for v = 1:length(UBrxnList)
         if (model.lb(v) < 0) & (model.ub(v) > 0) & (NullSpace(v,z) ~= 0)
           TIC = UBrxnList(v); % Extreme pathways formed by unbounded reactions (TICs)
           fprintf(fileID, '%s \n', TIC{1:end});
           rxncount = rxncount + 1;
         end   
         if (model.lb(v) < 0) && (model.ub(v) == 0) & (NullSpace(v,z) < 0)
           TIC = UBrxnList(v); % Extreme pathways formed by unbounded reactions (TICs)
           fprintf(fileID, '%s \n', TIC{1:end});
           rxncount = rxncount + 1;
         end
         if (model.lb == 0) & (model.ub(v) > 0) & (NullSpace(v,z) > 0)
           TIC = UBrxnList(v); % Extreme pathways formed by unbounded reactions (TICs)
           fprintf(fileID, '%s \n', TIC{1:end});
           rxncount = rxncount + 1;
         end    
     end
     if (rxncount > 1)
          TIC_count = TIC_count + 1;
          fprintf(fileID, 'TIC_#%d.\n', TIC_count); % No. of TIC
     elseif (rxncount == 0)
          fprintf(fileID, '#####THIS IS NOT A TIC#####\n');
     else 
          fprintf(fileID, '#####THIS IS NOT A TIC#####\n'); % That's not a TIC
         
     end
 end 
fprintf(fileID, '###########\n');
fclose(fileID); % closing file 

for i = 1:length(model.rxns)
    
    if (minFVA(i) < -50) && (maxFVA(i) > 50)
      
      count_minmax = count_minmax + 1;
        
    end   
    
end

count_UR = (count_minFVA - count_minmax) + (count_maxFVA - count_minmax) + count_minmax

NURs(1,1) = count_UR;
NURs(1,2) = (count_UR/length(model.rxns)*100);

%% Summary Results 

Number_of_Unbounded_Reactions = table (NURs(1,1),NURs(1,2),'RowNames',{'sclav_red'},'VariableNames',{'Unbounded_Rxns','Percentage_URs'})
file_excel = 'Results_UBs.xlsx';
writetable(Number_of_Unbounded_Reactions,file_excel,'Sheet',1,'WriteRowNames',true)