%% Model Reduce Validation - Strategy -> maximize growth and relaxed experimental constraints (lower bounds CO2, O2, Glycerol)
%% Draft Model Reduced
% author: Victor A. Lopez-Agudelo, University of Valle
clc;
%clear all;
%addpath(genpath('G:\Mi unidad\0_POSTDOC-UNIVALLE\RAVEN-master'))
% Carga el modelo de  S. clavuligerus. Descargado de:
% https://www.mdpi.com/2076-2607/8/9/1255
% excel file containing the model with the subsystems we want to include in the reduced model.
% load excel files for each model reduction iteration: sclav_red_0,sclav_red_1, sclav_red_2, sclav_red_3...


filename = 'sclav_red_draft_%d.xlsx';

inputask = inputdlg('Enter number of reduced model iteration: example inputs 0,1,2,3,...',...
                    'Sclav Reduced Model',[1 20])

user_input = str2double(inputask{1}); %converting to double user input

%modelSclav    = importExcelModel(sprintf(filename,user_input),true);
sclav_red = importExcelModel(sprintf(filename,user_input),true);
sclav_large = importExcelModel('iDG1237_raven_subsystem-01112020.xlsx');

%% Solution FBA
%SolverOK = changeCobraSolver('ibm_cplex','all');

%% Agregar datos acerca de S.clavuligerus
sclav_red.id     = 'sclav_atcc_27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'L�pez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'L�pez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';



exchangeRxns_red = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
model = changeRxnBounds(sclav_red,exchangeRxns_red,0,'l'); % define medium unlimited
% defining medium for sclav
medium_base = {'EX_h2o_e','EX_h_e','EX_cl_e','EX_nh4_e','EX_k_e','EX_ca2_e','EX_cit_e','EX_mg2_e','EX_mn2_e',...
                'EX_cobalt2_e','EX_zn2_e','EX_cu2_e','EX_fe2_e', 'EX_fe3_e','EX_mobd_e','EX_so4_e', 'EX_cbl1_e', 'EX_ni2_e'};

medium_trace = {'EX_glu__L_e'};

oxygen = 'EX_o2_e';
phosphate = 'EX_pi_e';
co2 = 'EX_co2_e';
glycerol = 'EX_glyc_e';
citrate = 'EX_cit_e';
clavulanate = 'EX_clav_e';

% constraining sclav reduced model
model = changeRxnBounds(model,medium_trace,-0.35,'l');
model = changeRxnBounds(model,medium_base,-1000,'l');
model = changeRxnBounds(model,oxygen,-1.665,'l');
model = changeRxnBounds(model,phosphate,0,'l');
model = changeRxnBounds(model,glycerol,-1.110,'l');
model = changeRxnBounds(model,co2,0.067,'u');
model = changeRxnBounds(model,citrate,0,'l');
%model = changeRxnBounds(model,'ATPM',3.02,'b'); % NGAM

% constraining sclav large model iDG1237
exchangeRxns_large = sclav_large.rxns(findExcRxns(sclav_large)); % exchange reactions
model2 = changeRxnBounds(sclav_large,exchangeRxns_large,0,'l'); % define medium unlimited

model2 = changeRxnBounds(model2,medium_trace,-0.35,'l');
model2 = changeRxnBounds(model2,medium_base,-1000,'l');
model2 = changeRxnBounds(model2,oxygen,-1.665,'l');
model2 = changeRxnBounds(model2,phosphate,0,'l');
model2 = changeRxnBounds(model2,glycerol,-1.110,'l');
model2 = changeRxnBounds(model2,co2,0.067,'u');
model2 = changeRxnBounds(model2,citrate,0,'l');
%model2 = changeRxnBounds(model2,'ATPM',3.02,'b'); % NGAM


solutionFBA = optimizeCbModel(model,'max',1);
[minFVA_red,maxFVA_red] = fluxVariability(model,100);

solutionFBA_sclav_large = optimizeCbModel(model2,'max',1);
[minFVA_large,maxFVA_large] = fluxVariability(model2,100);

Rates_Names = {'Growth Rate';oxygen;glycerol;phosphate;co2;clavulanate;'EX_glu__L_e'};

Rates_Sclav_red_minFVA = [minFVA_red(findRxnIDs(model,'Growth'));minFVA_red(findRxnIDs(model,oxygen));minFVA_red(findRxnIDs(model,glycerol));...
                   minFVA_red(findRxnIDs(model,phosphate));minFVA_red(findRxnIDs(model,co2));minFVA_red(findRxnIDs(model,clavulanate));...
                   minFVA_red(findRxnIDs(model,'EX_glu__L_e'))];

Rates_Sclav_red_maxFVA = [maxFVA_red(findRxnIDs(model,'Growth'));maxFVA_red(findRxnIDs(model,oxygen));maxFVA_red(findRxnIDs(model,glycerol));...
                   maxFVA_red(findRxnIDs(model,phosphate));maxFVA_red(findRxnIDs(model,co2));maxFVA_red(findRxnIDs(model,clavulanate));...
                   maxFVA_red(findRxnIDs(model,'EX_glu__L_e'))];


Rates_Sclav_large_minFVA = [minFVA_large(findRxnIDs(model2,'Growth'));minFVA_large(findRxnIDs(model2,oxygen));minFVA_large(findRxnIDs(model2,glycerol));...
                   minFVA_large(findRxnIDs(model2,phosphate));minFVA_large(findRxnIDs(model2,co2));minFVA_large(findRxnIDs(model2,clavulanate));...
                   minFVA_large(findRxnIDs(model2,'EX_glu__L_e'))];

Rates_Sclav_large_maxFVA = [maxFVA_large(findRxnIDs(model2,'Growth'));maxFVA_large(findRxnIDs(model2,oxygen));maxFVA_large(findRxnIDs(model2,glycerol));...
                   maxFVA_large(findRxnIDs(model2,phosphate));maxFVA_large(findRxnIDs(model2,co2));maxFVA_large(findRxnIDs(model2,clavulanate));...
                   maxFVA_large(findRxnIDs(model2,'EX_glu__L_e'))];

Rates_Sclav_red_FBAminNorm = [solutionFBA.v(findRxnIDs(model,'Growth'));solutionFBA.v(findRxnIDs(model,oxygen));solutionFBA.v(findRxnIDs(model,glycerol));...
                   solutionFBA.v(findRxnIDs(model,phosphate));solutionFBA.v(findRxnIDs(model,co2));solutionFBA.v(findRxnIDs(model,clavulanate));...
                   solutionFBA.v(findRxnIDs(model,'EX_glu__L_e'))];

Rates_Sclav_large_FBAminNorm = [solutionFBA_sclav_large.v(findRxnIDs(model2,'Growth'));solutionFBA_sclav_large.v(findRxnIDs(model2,oxygen));solutionFBA_sclav_large.v(findRxnIDs(model2,glycerol));...
                   solutionFBA_sclav_large.v(findRxnIDs(model2,phosphate));solutionFBA_sclav_large.v(findRxnIDs(model2,co2));solutionFBA_sclav_large.v(findRxnIDs(model2,clavulanate));...
                   solutionFBA_sclav_large.v(findRxnIDs(model2,'EX_glu__L_e'))];

Simulated_Rates = table(Rates_Names,Rates_Sclav_red_minFVA,Rates_Sclav_red_maxFVA,Rates_Sclav_large_minFVA,Rates_Sclav_large_maxFVA,Rates_Sclav_red_FBAminNorm,Rates_Sclav_large_FBAminNorm,...
                       'VariableName',{'Rates_Names','minFVA_sclav_red','maxFVA_sclav_red','minFVA_sclav_large','maxFVA_sclav_large','Sclav_Red_FBAminNorm','Sclav_large_FBAminNorm'});

%% Print Vector
% printFluxVector(model, minFVA_red, true, true)
% printFluxVector(model, maxFVA_red, true, true)
% printFluxVector(model2, minFVA_large, true, true)
% printFluxVector(model2, maxFVA_large, true, true)


% Robustness Analysis
figure()
% Glutamate Effect on Growth Rate and CA production in Sclav
model = changeRxnBounds(model,'EX_glu__L_e',-20,'l');
[controlFlux, objFlux] = robustnessAnalysis(model,'EX_glu__L_e',100);

figure()
model2 = changeRxnBounds(model2,'EX_glu__L_e',-20,'l');
[controlFlux2, objFlux2] = robustnessAnalysis(model2,'EX_glu__L_e',100);
