% Computation of Shadow Prices - Experiment 2 - Fed-batch, 48 h
% author: Victor A. Lopez-Agudelo, University of Valle
clc;
%clear all;
%addpath(genpath('G:\Mi unidad\0_POSTDOC-UNIVALLE\RAVEN-master'))
% Carga el modelo de  S. clavuligerus. Descargado de:
% https://www.mdpi.com/2076-2607/8/9/1255
% excel file containing the model with the subsystems we want to include in the reduced model.
% load excel files for each model reduction iteration: sclav_red_0,sclav_red_1, sclav_red_2, sclav_red_3...


filename = 'sclav_red_draft_%d.xlsx';

inputask = inputdlg('Enter number of reduced model iteration: example inputs 0,1,2,3,...',...
                    'Sclav Reduced Model',[1 20])

user_input = str2double(inputask{1}); %converting to double user input

%modelSclav    = importExcelModel(sprintf(filename,user_input),true);
sclav_red = importExcelModel(sprintf(filename,user_input),true);
sclav_large = importExcelModel('iDG1237_raven_subsystem-01112020.xlsx');

%% Solution FBA
SolverOK = changeCobraSolver('ibm_cplex','all');
%SolverOK = changeCobraSolver('glpk','LP');
%% Agregar datos acerca de S.clavuligerus
sclav_red.id     = 'sclav_atcc_27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'L�pez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';
sclav_red.annotation.defaultLB    = -1000;
sclav_red.annotation.defaultUB    = +1000;
sclav_red.annotation.taxonomy     = 'taxonomy/1901';
sclav_red.annotation.givenName    = 'Victor A.';
sclav_red.annotation.familyName   = 'L�pez-Agudelo';
sclav_red.annotation.email        = 'valonso.lopez@udea.edu.co';
sclav_red.annotation.organization = 'Universidad del Valle';
sclav_red.annotation.note         = 'Streptomyces clavuligerus ATCC 27064';
sclav_red.description             = 'Metabolic model of Streptomyces clavuligerus ATCC 27064';



exchangeRxns_red = sclav_red.rxns(findExcRxns(sclav_red)); % exchange reactions
model = changeRxnBounds(sclav_red,exchangeRxns_red,0,'l'); % define medium unlimited
model = changeObjective(model,'EX_clav_e'); % changing objective function to clavulanic acid
% defining medium for sclav
medium_base = {'EX_h2o_e','EX_h_e','EX_cl_e','EX_nh4_e','EX_k_e','EX_ca2_e','EX_cit_e','EX_mg2_e','EX_mn2_e',...
                'EX_cobalt2_e','EX_zn2_e','EX_cu2_e','EX_fe2_e', 'EX_fe3_e','EX_mobd_e','EX_so4_e', 'EX_cbl1_e', 'EX_ni2_e'};

medium_trace = {'EX_glu__L_e'};

oxygen = 'EX_o2_e';
phosphate = 'EX_pi_e';
co2 = 'EX_co2_e';
glycerol = 'EX_glyc_e';
citrate = 'EX_cit_e';
clavulanate = 'EX_clav_e';

% constraining sclav reduced model
model = changeRxnBounds(model,medium_trace,-0.35,'l');
model = changeRxnBounds(model,medium_base,-1000,'l');
model = changeRxnBounds(model,oxygen,-1.2,'l');
model = changeRxnBounds(model,phosphate,-10,'l');
model = changeRxnBounds(model,phosphate,0,'u');
model = changeRxnBounds(model,glycerol,-0.470,'l');
model = changeRxnBounds(model,co2,1.4,'u');
model = changeRxnBounds(model,citrate,0,'l');
model = changeRxnBounds(model,clavulanate,0.004,'u');
model = changeRxnBounds(model,'EX_succ_e',0.007,'l');
model = changeRxnBounds(model,'EX_oaa_e',0.003,'u');
%model = changeRxnBounds(model,'EX_oaa_e',1000,'u');
model = changeRxnBounds(model,'EX_mal__L_e',0,'u');
model = changeRxnBounds(model,'EX_pyr_e',0.005,'l');
%model = changeRxnBounds(model,'EX_pyr_e',1000,'u');
model = changeRxnBounds(model,'EX_ac_e',0,'u');
model = changeRxnBounds(model,'Growth',0.031, 'b');
%model = changeRxnBounds(model,'ATPM',3.02,'b'); % NGAM

% constraining sclav large model iDG1237
exchangeRxns_large = sclav_large.rxns(findExcRxns(sclav_large)); % exchange reactions
model2 = changeRxnBounds(sclav_large,exchangeRxns_large,0,'l'); % define medium unlimited
model2 = changeObjective(model2,'EX_clav_e'); % changing objective function to clavulanic acid

model2 = changeRxnBounds(model2,medium_trace,-0.35,'l');
model2 = changeRxnBounds(model2,medium_base,-1000,'l');
model2 = changeRxnBounds(model2,oxygen,-1.2,'l');
model2 = changeRxnBounds(model2,phosphate,-10,'l');
model2 = changeRxnBounds(model2,phosphate,0,'u');
model2 = changeRxnBounds(model2,glycerol,-0.470,'l');
model2 = changeRxnBounds(model2,co2,1.4,'u');
model2 = changeRxnBounds(model2,citrate,0,'l');
model2 = changeRxnBounds(model2,clavulanate,0.004,'u'); % for shadow prices calculation "clavulanate, 0.0, 'l'".
model2 = changeRxnBounds(model2,'EX_succ_e',0.007,'l');
model2 = changeRxnBounds(model2,'EX_oaa_e',0.003,'u');
%model2 = changeRxnBounds(model2,'EX_oaa_e',1000,'u');
model2 = changeRxnBounds(model2,'EX_mal__L_e',0,'u');
model2 = changeRxnBounds(model2,'EX_pyr_e',0.005,'l');
model2 = changeRxnBounds(model2,'EX_pyr_e',1000,'u');
model2 = changeRxnBounds(model2,'EX_ac_e',0,'u');
model2 = changeRxnBounds(model2,'Growth',0.031, 'b');
%model2 = changeRxnBounds(model2,'ATPM',3.02,'b'); % NGAM


solutionFBA = optimizeCbModel(model,'max',1);
[minFVA_red,maxFVA_red] = fluxVariability(model,100);

solutionFBA_sclav_large = optimizeCbModel(model2,'max',1);
[minFVA_large,maxFVA_large] = fluxVariability(model2,100);

Rates_Names = {'Growth Rate';oxygen;glycerol;phosphate;co2;clavulanate;'EX_glu__L_e';'EX_succ_e';'EX_oaa_e';'EX_mal__L_e';'EX_pyr_e';'EX_ac_e'};

Rates_Sclav_red_minFVA = [minFVA_red(findRxnIDs(model,'Growth'));minFVA_red(findRxnIDs(model,oxygen));minFVA_red(findRxnIDs(model,glycerol));...
                   minFVA_red(findRxnIDs(model,phosphate));minFVA_red(findRxnIDs(model,co2));minFVA_red(findRxnIDs(model,clavulanate));...
                   minFVA_red(findRxnIDs(model,'EX_glu__L_e'));minFVA_red(findRxnIDs(model,'EX_succ_e'));minFVA_red(findRxnIDs(model,'EX_oaa_e'));...
                   minFVA_red(findRxnIDs(model,'EX_mal__L_e')); minFVA_red(findRxnIDs(model,'EX_pyr_e'));minFVA_red(findRxnIDs(model,'EX_ac_e'))];

Rates_Sclav_red_maxFVA = [maxFVA_red(findRxnIDs(model,'Growth'));maxFVA_red(findRxnIDs(model,oxygen));maxFVA_red(findRxnIDs(model,glycerol));...
                   maxFVA_red(findRxnIDs(model,phosphate));maxFVA_red(findRxnIDs(model,co2));maxFVA_red(findRxnIDs(model,clavulanate));...
                   maxFVA_red(findRxnIDs(model,'EX_glu__L_e'));maxFVA_red(findRxnIDs(model,'EX_succ_e'));maxFVA_red(findRxnIDs(model,'EX_oaa_e'));...
                   maxFVA_red(findRxnIDs(model,'EX_mal__L_e')); maxFVA_red(findRxnIDs(model,'EX_pyr_e'));maxFVA_red(findRxnIDs(model,'EX_ac_e'))];


Rates_Sclav_large_minFVA = [minFVA_large(findRxnIDs(model2,'Growth'));minFVA_large(findRxnIDs(model2,oxygen));minFVA_large(findRxnIDs(model2,glycerol));...
                   minFVA_large(findRxnIDs(model2,phosphate));minFVA_large(findRxnIDs(model2,co2));minFVA_large(findRxnIDs(model2,clavulanate));...
                   minFVA_large(findRxnIDs(model2,'EX_glu__L_e'));minFVA_large(findRxnIDs(model,'EX_succ_e'));minFVA_large(findRxnIDs(model,'EX_oaa_e'));...
                   minFVA_large(findRxnIDs(model,'EX_mal__L_e'));minFVA_large(findRxnIDs(model,'EX_pyr_e'));minFVA_large(findRxnIDs(model,'EX_ac_e'))];

Rates_Sclav_large_maxFVA = [maxFVA_large(findRxnIDs(model2,'Growth'));maxFVA_large(findRxnIDs(model2,oxygen));maxFVA_large(findRxnIDs(model2,glycerol));...
                   maxFVA_large(findRxnIDs(model2,phosphate));maxFVA_large(findRxnIDs(model2,co2));maxFVA_large(findRxnIDs(model2,clavulanate));...
                   maxFVA_large(findRxnIDs(model2,'EX_glu__L_e'));maxFVA_large(findRxnIDs(model,'EX_succ_e'));maxFVA_large(findRxnIDs(model,'EX_oaa_e'));...
                   maxFVA_large(findRxnIDs(model,'EX_mal__L_e'));maxFVA_large(findRxnIDs(model,'EX_pyr_e'));maxFVA_large(findRxnIDs(model,'EX_ac_e'))];

Rates_Sclav_red_FBAminNorm = [solutionFBA.v(findRxnIDs(model,'Growth'));solutionFBA.v(findRxnIDs(model,oxygen));solutionFBA.v(findRxnIDs(model,glycerol));...
                   solutionFBA.v(findRxnIDs(model,phosphate));solutionFBA.v(findRxnIDs(model,co2));solutionFBA.v(findRxnIDs(model,clavulanate));...
                   solutionFBA.v(findRxnIDs(model,'EX_glu__L_e'));solutionFBA.v(findRxnIDs(model,'EX_succ_e'));solutionFBA.v(findRxnIDs(model,'EX_oaa_e'));...
                   solutionFBA.v(findRxnIDs(model,'EX_mal__L_e'));solutionFBA.v(findRxnIDs(model,'EX_pyr_e'));solutionFBA.v(findRxnIDs(model,'EX_ac_e'))];

Rates_Sclav_large_FBAminNorm = [solutionFBA_sclav_large.v(findRxnIDs(model2,'Growth'));solutionFBA_sclav_large.v(findRxnIDs(model2,oxygen));solutionFBA_sclav_large.v(findRxnIDs(model2,glycerol));...
                   solutionFBA_sclav_large.v(findRxnIDs(model2,phosphate));solutionFBA_sclav_large.v(findRxnIDs(model2,co2));solutionFBA_sclav_large.v(findRxnIDs(model2,clavulanate));...
                   solutionFBA_sclav_large.v(findRxnIDs(model2,'EX_glu__L_e'));solutionFBA_sclav_large.v(findRxnIDs(model,'EX_succ_e'));solutionFBA_sclav_large.v(findRxnIDs(model,'EX_oaa_e'));...
                   solutionFBA_sclav_large.v(findRxnIDs(model,'EX_mal__L_e'));solutionFBA_sclav_large.v(findRxnIDs(model,'EX_pyr_e'));solutionFBA_sclav_large.v(findRxnIDs(model,'EX_ac_e'))];

Simulated_Rates = table(Rates_Names,Rates_Sclav_red_minFVA,Rates_Sclav_red_maxFVA,Rates_Sclav_large_minFVA,Rates_Sclav_large_maxFVA,Rates_Sclav_red_FBAminNorm,Rates_Sclav_large_FBAminNorm,...
                       'VariableName',{'Rates_Names','minFVA_sclav_red','maxFVA_sclav_red','minFVA_sclav_large','maxFVA_sclav_large','Sclav_Red_FBAminNorm','Sclav_large_FBAminNorm'});
save('0_SP_results/Simulated_Rates_scenario_2.mat','Simulated_Rates') % Saving Global Simulated Rates
%% FVA fluxes table in sclav_red
FVA_table = table(minFVA_red, maxFVA_red,solutionFBA.v,'VariableNames',{'minFVA','maxFVA','minNormFBA'});
save('0_SP_results/FVA_red_scenario_2.mat','FVA_table') % Saving Global Simulated Rates

%% Uniform Sampling
options.nStepsPerPoint = 1;
options.nPointsReturned = 500;
options.toRound = 1;
% sclav_red
[R, X] = sampleCbModel(model, [], [], options);

%% Model is severely undersampled therefore:
% nStepsPerPoint
options.nStepsPerPoint = 8 * size(R.A, 2);
options.nPointsReturned = 5000;
% We can avoid the rounding step by inputting the rounded polytope
% from the previous round of sampling.
options.toRound = 0;

[~, X] = sampleCbModel(model, [], [],options,R);

% Saving Sampling data
save('0_SP_results/sclav_red_uniform_sampling_scenario_2.mat','X'); % saving sampling data

%% Shadow Prices
% Shadow Prices were computed by computing a traditional FBA where
% clavulanate production was not constrained in the upperbound.

% sclav_red
model_sp = changeRxnBounds(model,clavulanate,0.0,'l'); % not upper constrained clavulanate
model_sp = changeRxnBounds(model_sp,clavulanate,1000,'u'); % not upper constrained clavulanate
model_sp = changeObjective(model_sp,'EX_clav_e'); % changing objective function to clavulanic acid
solutionFBA_simplex = optimizeCbModel(model_sp,'max');
CPLEX_ShadowPrices_sclav_red = table(model_sp.mets,solutionFBA_simplex.y,'VariableName',{'Metabolites','ShadowPrices'});
CPLEX_ReducedCosts_sclav_red = table(model_sp.rxns,solutionFBA_simplex.w,'VariableName',{'Reactions','ReducedCost'});
% Print Data
save('0_SP_results/sclav_red_shadowPrices_scenario_2.mat','CPLEX_ShadowPrices_sclav_red') % Saving shadow prices
save('0_SP_results/sclav_red_reducedcosts_scenario_2.mat','CPLEX_ReducedCosts_sclav_red') % Saving reduced costs

% % sclav_large
% model2 = changeRxnBounds(model2,clavulanate,0.0,'l'); % not upper constrained clavulanate
% model2 = changeRxnBounds(model2,clavulanate,1000,'u'); % not upper constrained clavulanate
% solutionFBA_sclav_large_simplex = optimizeCbModel(model2,'max');
% ShadowPrices_sclav_large = table(model2.mets,solutionFBA_sclav_large_simplex.y,'VariableName',{'Metabolites','ShadowPrices'});
% ReducedCosts_sclav_large = table(model2.rxns,solutionFBA_sclav_large_simplex.w,'VariableName',{'Reactions','ReducedCost'});
% % Print Data
% save('0_SP_results/sclav_large_shadowPrices_scenario_2.mat','ShadowPrices_sclav_large') % Saving shadow prices
% save('0_SP_results/sclav_large_reducedcosts_scenario_2.mat','ReducedCosts_sclav_large') % Saving reduced costs

%% Shadow Prices Computation
solutionFBA_default = optimizeCbModel(model,'max');

model_p = model;
model_n = model;
sp_incremental = zeros(length(model.mets),1); % empty vector for saving incremental shadow prices.
sp_decremental = zeros(length(model.mets),1); % empty vector for saving decremental shadow prices.
origStatText_p = cell(length(model.mets),1); % empty vector for saving optimization status text for incremental shadow prices
origStatText_n = cell(length(model.mets),1); % empty vector for saving optimization status text for decremental shadow prices

% computing incremental shadow prices
for i = 1:length(model.mets)
    model_p = model;
    model_p.b(i) = 0.001; % incremental shadow price of 1.
    solutionFBA_p = optimizeCbModel(model_p,'max'); % FBA
    origStatText_p{i} = solutionFBA_p.origStatText;
    sp_incremental(i) = ((solutionFBA_p.f-solutionFBA_default.f)/model_p.b(i));
end
% computing decremental shadow prices
SP_p = table(origStatText_p,sp_incremental,'VariableNames',{'OptimizationStatus','ShadowPriceValue'});
for i = 1:length(model.mets)
    model_n = model;
    model_n.b(i) = -0.001; % decremental shadow price of 1.
    solutionFBA_n = optimizeCbModel(model_n,'max'); % FBA
    origStatText_n{i} = solutionFBA_n.origStatText;
    sp_decremental(i) = ((solutionFBA_n.f-solutionFBA_default.f)/model_n.b(i));
end
SP_n = table(origStatText_n,sp_decremental,'VariableNames',{'OptimizationStatus','ShadowPriceValue'});
